/**
 * @fileoverview test wysiwyg decrease depth command
 * @author NHN Ent. FE Development Lab <dl_javascript@nhnent.com>
 */
import $ from 'jquery';

import DecreaseDepth from '../../src/js/wysiwygCommands/decreaseDepth';
import WwTaskManager from '../../src/js/wwTaskManager';
import WysiwygEditor from '../../src/js/wysiwygEditor';
import EventManager from '../../src/js/eventManager';

describe('DecreaseDepth', () => {
  let wwe, sq, container;

  beforeEach(() => {
    container = document.createElement('div');
    document.body.appendChild(container);

    wwe = new WysiwygEditor($(container), new EventManager());
    wwe.init();

    sq = wwe.getEditor();
    wwe.componentManager.addManager(WwTaskManager);
    sq.focus();

    wwe.get$Body().html(`
            <ul>
                <li data-te-task class="task-list-item">abcdef</li>
                <ul>
                    <li data-te-task class="task-list-item checked">abcde</li>
                </ul>
                <li data-te-task class="task-list-item"> </li>
            </ul>
        `);
  });

  // we need to wait squire input event process
  afterEach(done => {
    setTimeout(() => {
      document.body.removeChild(container);
      done();
    });
  });

  it('should be able to decrease depth second to first.', () => {
    const range = wwe.getEditor().getSelection().cloneRange();

    range.setStart(wwe.get$Body().find('li')[1].firstChild, 0);
    range.collapse(true);

    sq.setSelection(range);

    DecreaseDepth.exec(wwe);

    expect(sq.get$Body().find('ul > li').length).toEqual(3);
    expect(sq.get$Body().find('ul > ul > li').length).toEqual(0);
    expect(sq.get$Body().find('ul > li').hasClass('task-list-item')).toBe(true);
    expect(sq.get$Body().find('ul > li').hasClass('checked')).toBe(true);
  });
  it('should break out list element and delete input.', () => {
    const range = wwe.getEditor().getSelection().cloneRange();

    range.setStart(wwe.get$Body().find('li')[2].firstChild, 0);
    range.collapse(true);

    sq.setSelection(range);

    DecreaseDepth.exec(wwe);

    expect(sq.get$Body().find('ul li').length).toEqual(2);
    expect(sq.get$Body().find('ul ul li').length).toEqual(1);
    expect(sq.get$Body().find('ul li').hasClass('task-list-item')).toBe(true);
  });
  it('should preserve original class(task / non-task)', () => {
    const range = wwe.getEditor().getSelection().cloneRange();

    wwe.get$Body().html(`
            <ul>
                <li data-te-task class="task-list-item">abcdef</li>
                <ul>
                    <li>abcde</li>
                </ul>
                <li data-te-task class="task-list-item"> </li>
            </ul>
        `);

    range.setStart(wwe.get$Body().find('ul ul li')[0].firstChild, 0);
    range.collapse(true);

    sq.setSelection(range);

    DecreaseDepth.exec(wwe);

    const $Body = sq.get$Body();

    expect($Body.find('ul li').length).toEqual(3);
    expect($Body.find('ul ul li').length).toEqual(0);
    expect($Body.find('ul li').eq(0).hasClass('task-list-item')).toBe(true);
    expect($Body.find('ul li').eq(1).hasClass('task-list-item')).toBe(false);
    expect($Body.find('ul li').eq(2).hasClass('task-list-item')).toBe(true);
  });

  it('should not decrease if next element is UL/OL (arbitrary list)', () => {
    const range = wwe.getEditor().getSelection().cloneRange();

    wwe.get$Body().html(`
            <ul>
                <li>abcdef</li>
                <ul>
                    <li id="target">abcde</li>
                    <ul>
                        <li>abcde</li>
                    </ul>
                </ul>
            </ul>
        `);

    range.setStart(wwe.get$Body().find('#target')[0].firstChild, 0);
    range.collapse(true);

    sq.setSelection(range);

    DecreaseDepth.exec(wwe);

    const $Body = sq.get$Body();

    expect($Body.find('> ul > li').length).toEqual(1);
    expect($Body.find('> ul > ul > li').length).toEqual(1);
    expect($Body.find('> ul > ul > ul > li').length).toEqual(1);
  });

  describe('should decrease depth when cursor', () => {
    it('at startOffset 0.', () => {
      const range = wwe.getEditor().getSelection().cloneRange();

      range.setStart(wwe.get$Body().find('li')[1].firstChild, 0);
      range.collapse(true);

      sq.setSelection(range);

      DecreaseDepth.exec(wwe);

      expect(sq.get$Body().find('ul > li').length).toEqual(3);
      expect(sq.get$Body().find('ul ul li').length).toEqual(0);
      expect(sq.get$Body().find('ul > li').hasClass('task-list-item')).toBe(true);
    });
    it('should decrease depth when cursor at any offset.', () => {
      const range = wwe.getEditor().getSelection().cloneRange();

      range.setStart(wwe.get$Body().find('li')[1].firstChild, 2);
      range.collapse(true);

      sq.setSelection(range);

      DecreaseDepth.exec(wwe);

      expect(sq.get$Body().find('ul > li').length).toEqual(3);
      expect(sq.get$Body().find('ul ul li').length).toEqual(0);
      expect(sq.get$Body().find('ul > li').hasClass('task-list-item')).toBe(true);
    });
  });
});
